﻿using HarmonyLib;
using Steamworks;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using UltimateWater;
using UnityEngine;
using UnityEngine.UI;
using System.Reflection.Emit;
using HMLLibrary;
using RaftModLoader;

public class AugmentedEquipment : Mod
{
    private Harmony harmony;
    private string instanceId = "com.bahamut.augmentedequipment";

    private List<Item_Base> addedItems = new List<Item_Base>();

    private static float globalGatherSpeedMultiplier = 0f;
    private static int maxHotbarSlotIndex = 9;
    private static Vector3 originalInventoryVector = new Vector3(0f, 0f, 0f);
    private static Dictionary<EquipSlotType, string> equippedItemsCache = new Dictionary<EquipSlotType, string>
    {
        [CustomEquipSlotType.Get("None")] = null,
        [CustomEquipSlotType.Get("Feet")] = null,
        [CustomEquipSlotType.Get("SwimFeet")] = null,
        [CustomEquipSlotType.Get("Head")] = null,
        [CustomEquipSlotType.Get("Chest")] = null,
        [CustomEquipSlotType.Get("Backpack")] = null,
        [CustomEquipSlotType.Get("ZiplineTool")] = null,
        [CustomEquipSlotType.Get("Glove")] = null,
        [CustomEquipSlotType.Get("Belt")] = null,
        [CustomEquipSlotType.Get("OxygenBottle")] = null
    };

    public void Start()
    {
        LoadAugmentedEquipments();

        harmony = new Harmony(instanceId);
        harmony.PatchAll(Assembly.GetExecutingAssembly());
        Log("Mod has loaded!");
    }

    public override void WorldEvent_WorldUnloaded()
    {
        foreach (var s in equippedItemsCache.Keys.ToArray())
            equippedItemsCache[s] = null;
    }

    // Helper Methods
    private void LoadAugmentedEquipments()
    {
        foreach (KeyValuePair<string, ItemTemplate> kvp in AugmentedEquipmentItems.AllAugmentedEquipmentItems)
        {
            ItemTemplate itemTemplate = kvp.Value;
            if (itemTemplate.uniqueName != null)
            {
                CreateEquipment(
                  itemTemplate.baseItemReference,
                  itemTemplate.uniqueName,
                  itemTemplate.displayName,
                  itemTemplate.uniqueIndex,
                  itemTemplate.description,
                  itemTemplate.maxUses,
                  itemTemplate.maxStacks,
                  itemTemplate.equipSlotType,
                  itemTemplate.subCategory,
                  itemTemplate.subCategoryOrder,
                  itemTemplate.recipe,
                  itemTemplate.spriteName
                );
            }
            else
            {
                Item_Base item = ItemManager.GetItemByName(kvp.Key);
                if (itemTemplate.equipSlotType != item.settings_equipment.EquipType)
                {
                    ItemInstance_Equipment equipmentSlotType = new ItemInstance_Equipment(itemTemplate.equipSlotType);
                    Traverse.Create(item).Field("settings_equipment").SetValue(equipmentSlotType);
                }
                ItemInstance_Recipe itemInstanceRecipe =  new ItemInstance_Recipe(CraftingCategory.Equipment, item.settings_recipe.Learned,item.settings_recipe.LearnedFromBeginning, itemTemplate.subCategory, itemTemplate.subCategoryOrder);
                itemInstanceRecipe.NewCost = item.settings_recipe.NewCost;
                item.settings_recipe = itemInstanceRecipe;
            }
        }
    }

    private void CreateEquipment(string baseItemName, string uniqueName, string displayName, int uniqueIndex, string description, int maxUses, int maxStacks, EquipSlotType slotType, string subCategory, int subCategoryOrder, Dictionary<string, int> recipe, string spriteName = null)
    {
        //string spriteAsset = (spriteName != null) ? spriteName : uniqueName;
        //Sprite augmentedItemSprite = asset.LoadAsset<Sprite>(spriteAsset);
        
        var t = new Texture2D(0,0);
        t.LoadImage(GetEmbeddedFileBytes("Assets/" + uniqueName + ".png"));
        var t2 = new Texture2D(t.width, t.width, t.format, false);
        t2.SetPixels(t.GetPixels(0));
        t2.Apply(true, true);
        Destroy(t);
        Sprite itemIcon = Sprite.Create(t2, new Rect(0, 0, t2.width, t2.height), Vector2.one * 0.5f);

        Item_Base baseEquipmentItem = ItemManager.GetItemByName(baseItemName);
        Item_Base newEquipmentItem = Instantiate(baseEquipmentItem);

        // Update the properties the item -- overriding the initialized property so we can set maxUses.
        if (slotType == EquipSlotType.Backpack)
        {
            Traverse.Create(newEquipmentItem).Property("UniqueName").SetValue(uniqueName);
            Traverse.Create(newEquipmentItem).Property("UniqueIndex").SetValue(uniqueIndex);
            ItemInstance_Inventory itemInstanceInventory = new ItemInstance_Inventory(null, "", 1);
            Traverse.Create(newEquipmentItem).Field("settings_Inventory").SetValue(itemInstanceInventory);

        }
        else
        {
            Traverse.Create(newEquipmentItem).Field("hasBeenInitialized").SetValue(false);
            newEquipmentItem.Initialize(uniqueIndex, uniqueName, maxUses);
        }
        // ItemInstance_Inventory existingInventoryInstance = newEquipmentItem.settings_Inventory;
        ItemInstance_Inventory newInventoryInstance = new ItemInstance_Inventory(itemIcon, "", maxStacks);
        newEquipmentItem.settings_Inventory = newInventoryInstance;
        Traverse.Create(newEquipmentItem).Field("settings_Inventory").Property("DisplayName").SetValue(displayName);
        Traverse.Create(newEquipmentItem).Field("settings_Inventory").Property("Description").SetValue(description);
        Traverse.Create(newEquipmentItem).Field("settings_Inventory").Property("Sprite").SetValue(itemIcon);
        Traverse.Create(newEquipmentItem).Field("settings_Inventory").Property("stackSize").SetValue(maxStacks);
        ItemInstance_Equipment equipmentSlotType = new ItemInstance_Equipment(slotType);
        Traverse.Create(newEquipmentItem).Field("settings_equipment").SetValue(equipmentSlotType);

        ItemInstance_Recipe itemInstanceRecipe;
        if (subCategory == null)
            itemInstanceRecipe = new ItemInstance_Recipe(baseEquipmentItem.settings_recipe.CraftingCategory, false, false, baseEquipmentItem.settings_recipe.SubCategory, subCategoryOrder);
        else
            itemInstanceRecipe = new ItemInstance_Recipe(baseEquipmentItem.settings_recipe.CraftingCategory, false, false, subCategory, subCategoryOrder);
        newEquipmentItem.settings_recipe = itemInstanceRecipe;
        
        // Create the recipe for the item.
        List<CostMultiple> itemRecipe = new List<CostMultiple>();
        foreach (KeyValuePair<string, int> kvp in recipe)
        {
            Item_Base item = ItemManager.GetItemByName(kvp.Key);
            CostMultiple itemRecipeQuantity = new CostMultiple(new Item_Base[] { item }, kvp.Value);
            itemRecipe.Add(itemRecipeQuantity);
        }
        newEquipmentItem.settings_recipe.NewCost = itemRecipe.ToArray();
        addedItems.Add(newEquipmentItem);

        // Debug.Log($"Successfully Added {newEquipmentItem.UniqueName}");
        RAPI.RegisterItem(newEquipmentItem);
    }

    private static bool CheckForEquipmentType(PlayerInventory playerInventory, Dictionary<string, float> modifiers, string equipSlotTypeString)
    {
        EquipSlotType equipSlotType = CustomEquipSlotType.Get(equipSlotTypeString);
        Slot_Equip slotEquip = playerInventory.GetEquipmentSlotFromEquipmentType(equipSlotType);

        if (slotEquip != null)
        {
            ItemInstance itemInstance = slotEquip.itemInstance;
            string itemUniqueName = itemInstance.UniqueName;
            equippedItemsCache[equipSlotType] = itemUniqueName;

            ItemTemplate itemTemplate = null;
            AugmentedEquipmentItems.AllAugmentedEquipmentItems.TryGetValue(itemUniqueName, out itemTemplate);
            if (itemTemplate != null)
            {
                Dictionary<string, float> itemModifiers = itemTemplate.modifiers;
                if (itemModifiers != null)
                {
                    foreach (KeyValuePair<string, float> kvp in itemModifiers)
                    {
                        modifiers[kvp.Key] += kvp.Value;
                    }
                }
                return true;
            }
        }
        equippedItemsCache[equipSlotType] = null;
        return false;
    }

    private static GameObject GetHookGameObject(Network_Player networkPlayer, string hookType)
    {
        Transform playerRightHandTransform = networkPlayer.rightHandParent;
        foreach (Transform rightHandTransform in playerRightHandTransform)
        {
            Hook hookScript = rightHandTransform.gameObject.GetComponent<Hook>();
            if (hookScript != null)
            {
                GameObject hookGameObject = rightHandTransform.gameObject;
                foreach (Transform hookObjectTransform in rightHandTransform)
                {
                    foreach (Transform hookModelTransform in hookObjectTransform)
                    {
                        if (hookModelTransform.gameObject.name.Contains("Model"))
                        {
                            MeshFilter meshFilter = hookModelTransform.gameObject.GetComponent<MeshFilter>();
                            if (meshFilter.mesh.ToString().Contains(hookType))
                            {
                                return hookGameObject;
                            }
                        }
                    }
                }
            }
        }
        return null;
    }

    private static void SetAxeButtonCooldown(float axeButtonCooldownMultiplier)
    {
        Item_Base axeItemBase = ItemManager.GetItemByName("Axe");
        ItemInstance_Usable itemInstanceUsable = axeItemBase.settings_usable;
        Traverse.Create(itemInstanceUsable).Field("useButtonCooldown").SetValue(OriginalEquipmentStats.Get("Axe", "useButtonCooldown") - (OriginalEquipmentStats.Get("Axe", "useButtonCooldown") * axeButtonCooldownMultiplier));
    }

    private static bool IsBackpackEmpty(PlayerInventory inventory)
    {
        using (List<Slot>.Enumerator enumerator = inventory.backpackSlots.GetEnumerator())
        {
            while (enumerator.MoveNext())
            {
                if (!enumerator.Current.IsEmpty)
                {
                    return false;
                }
            }
        }
        return true;
    }

    private void UnloadEquipment()
    {
        List<Item_Base> allItems = Traverse.Create(typeof(ItemManager)).Field("allAvailableItems").GetValue() as List<Item_Base>;
        foreach (Item_Base item in addedItems)
        {
            // Debug.Log($"Unloading Item {item.UniqueName}...");
            allItems.Remove(item);
        }
        addedItems.Clear();
        Traverse.Create(typeof(ItemManager)).Field("allAvailableItems").SetValue(allItems);
    }

    private PersonController GetLocalPersonController()
    {
        Network_Player networkPlayer = RAPI.GetLocalPlayer();
        return networkPlayer.PersonController;
    }


    private static void UpdateBackpackUI()
    {
        Dictionary<string, int> backpackMap = new Dictionary<string, int>
        {
            ["Backpack"] = 0,
            ["Duffel_Backpack"] = 1,
            ["Backpack_Advanced"] = 2,
            ["Tactical_Backpack"] = 3
        };

        Transform inventoryParent = GameObject.Find("InventoryParent").transform;
        Transform inventoryPlayer = inventoryParent.Find("Inventory_Player");
        string backpackItemCache = equippedItemsCache[CustomEquipSlotType.Get("Backpack")];
        int backpackTierMultiplier = (backpackItemCache != null) ? backpackMap[backpackItemCache] : 0;
        inventoryPlayer.localPosition = new Vector3(originalInventoryVector.x, originalInventoryVector.y + (35 * backpackTierMultiplier), 0);
    }

    private static void UpdatePlayerUI(bool isBeltEquipped)
    {
        Dictionary<string, int> beltMap = new Dictionary<string, int>
        {
            ["Thatch_Belt"] = 1,
            ["Tattered_Belt"] = 2,
            ["Leather_Belt"] = 3,
            ["Utility_Belt"] = 4,
            ["Caravan_Belt"] = 5
        };

        if (RAPI.GetLocalPlayer().IsLocalPlayer)
        {
            Transform inventoryParent = GameObject.Find("InventoryParent").transform;
            Transform hotbarParent = inventoryParent.Find("Hotbar");
            RectTransform hotbarParentRectTransform = hotbarParent.gameObject.GetComponent<RectTransform>();
            Rect hotbarParentRect = hotbarParentRectTransform.rect;
            string beltItemCache = equippedItemsCache[CustomEquipSlotType.Get("Belt")];
            int beltTierMultiplier = (beltItemCache != null) ? beltMap[beltItemCache] : 0;
            hotbarParentRectTransform.sizeDelta = new Vector2(OriginalEquipmentStats.Get("Hotbar", "width") + (70 * beltTierMultiplier), OriginalEquipmentStats.Get("Hotbar", "height"));
            Transform hotslotTransform = hotbarParent.Find("Hotslot parent");
            Slot[] hotSlots = hotslotTransform.GetComponentsInChildren<Slot>(true);
            for (int i = 10; i < 15; ++i)
            {
                hotSlots[i].gameObject.SetActiveSafe(false);
                hotSlots[i].active = false;
            }
            maxHotbarSlotIndex = 9;
            for (int i = 10; i < 10 + beltTierMultiplier; ++i)
            {
                hotSlots[i].gameObject.SetActiveSafe(true);
                hotSlots[i].active = true;
                maxHotbarSlotIndex = i;
            }
            Transform equipParent = inventoryParent.Find("Inventory_Player/SlotParent/Equipslot parent");
            GridLayoutGroup equipGridLayoutGroup = equipParent.gameObject.GetComponent<GridLayoutGroup>();
            Slot[] equipSlots = equipParent.GetComponentsInChildren<Slot>(true);
            for (int i = 5; i < 10; ++i)
            {
                equipSlots[i].gameObject.SetActiveSafe(false);
                equipSlots[i].active = false;
                equipGridLayoutGroup.constraintCount = 1;
            }
            for (int i = 5; i < 5 + beltTierMultiplier; ++i)
            {
                equipSlots[i].gameObject.SetActiveSafe(true);
                equipSlots[i].active = true;
                equipGridLayoutGroup.constraintCount = 2;
            }
        }
    }

    // Patches
    [HarmonyPatch(typeof(Slot_Equip), "SetItem", new[] { typeof(ItemInstance) })]
    class SlotEquipSetItem_Patch
    {
        private static void Postfix(Slot_Equip __instance, ItemInstance newInstance)
        {
            Network_Player networkPlayer = RAPI.GetLocalPlayer();
            PlayerInventory playerInventory = networkPlayer.Inventory;
            PersonController personController = networkPlayer.PersonController;
            PlayerStats playerStats = networkPlayer.Stats;
            GameObject scrapHook = GetHookGameObject(networkPlayer, "Scrap");
            Hook scrapHookScript = (scrapHook != null) ? scrapHook.GetComponent<Hook>() : null;
            GameObject titaniumHook = GetHookGameObject(networkPlayer, "Titanium");
            Hook titaniumHookScript = (titaniumHook != null) ? titaniumHook.GetComponent<Hook>() : null;

            Dictionary<string, float> modifiers = new Dictionary<string, float>
            {
                ["pullSpeedModifier"] = 0f,
                ["gatherTimeModifier"] = 0f,
                ["normalSpeedModifier"] = 0f,
                ["sprintSpeedModifier"] = 0f,
                ["swimSpeedModifier"] = 0f,
                ["axeButtonCooldownModifier"] = 0f,
                ["gatherSpeedModifier"] = 0f,
                ["oxygenLossModifier"] = 0f
            };

            bool isChestEquipped = CheckForEquipmentType(playerInventory, modifiers, "Chest");
            bool isFeetEquipped = CheckForEquipmentType(playerInventory, modifiers, "Feet");
            bool isSwimFeetEquipped = CheckForEquipmentType(playerInventory, modifiers, "SwimFeet");
            bool isGloveEquipped = CheckForEquipmentType(playerInventory, modifiers, "Glove");
            bool isBackpackEquipped = CheckForEquipmentType(playerInventory, modifiers, "Backpack");
            bool isHeadEquipped = CheckForEquipmentType(playerInventory, modifiers, "Head");
            bool isBeltEquipped = CheckForEquipmentType(playerInventory, modifiers, "Belt");
            bool isOxygenEquipped = CheckForEquipmentType(playerInventory, modifiers, "OxygenBottle");

            if (isBackpackEquipped)
            {
                if (newInstance != null)
                    switch (newInstance.UniqueName)
                    {
                        case "Backpack": playerInventory.SetBackpackActiveSlots(10); break;
                        case "Duffel_Backpack": playerInventory.SetBackpackActiveSlots(15); break;
                        case "Backpack_Advanced": playerInventory.SetBackpackActiveSlots(20); break;
                        case "Tactical_Backpack": playerInventory.SetBackpackActiveSlots(25); break;
                    }
            }
            else playerInventory.SetBackpackActiveSlots(0);
            UpdateBackpackUI();

            // Take the calculated modifiers and apply them to the Scrap Hook script.
            if (scrapHookScript != null)
            {
                scrapHookScript.pullSpeed = OriginalEquipmentStats.Get("ScrapHook", "pullSpeed") + (OriginalEquipmentStats.Get("ScrapHook", "pullSpeed") * modifiers["pullSpeedModifier"]);
                scrapHookScript.gatherTime = OriginalEquipmentStats.Get("ScrapHook", "gatherTime") - (OriginalEquipmentStats.Get("ScrapHook", "gatherTime") * modifiers["gatherTimeModifier"]);
            }
            if (titaniumHookScript != null)
            {
                titaniumHookScript.pullSpeed = OriginalEquipmentStats.Get("TitaniumHook", "pullSpeed") + (OriginalEquipmentStats.Get("TitaniumHook", "pullSpeed") * modifiers["pullSpeedModifier"]);
                titaniumHookScript.gatherTime = OriginalEquipmentStats.Get("TitaniumHook", "gatherTime") - (OriginalEquipmentStats.Get("TitaniumHook", "gatherTime") * modifiers["gatherTimeModifier"]);
            }

            // Take the calculated modifiers and apply them to the Player's Person Controller.
            personController.normalSpeed = OriginalEquipmentStats.Get("Player", "normalSpeed") + (OriginalEquipmentStats.Get("Player", "normalSpeed") * modifiers["normalSpeedModifier"]);
            personController.sprintSpeed = OriginalEquipmentStats.Get("Player", "sprintSpeed") + (OriginalEquipmentStats.Get("Player", "sprintSpeed") * modifiers["sprintSpeedModifier"]);
            personController.swimSpeed = OriginalEquipmentStats.Get("Player", "swimSpeed") + (OriginalEquipmentStats.Get("Player", "swimSpeed") * modifiers["swimSpeedModifier"]);

            // Take the calculated modifiers and apply them to the Player's Stats.
            playerStats.stat_oxygen.SetOxygenLostPerSecond(OriginalEquipmentStats.Get("OxygenBottle", "oxygenLoss") - (OriginalEquipmentStats.Get("OxygenBottle", "oxygenLoss") * modifiers["oxygenLossModifier"]));

            SetAxeButtonCooldown(modifiers["axeButtonCooldownModifier"]);
            globalGatherSpeedMultiplier = modifiers["gatherSpeedModifier"];

            UpdatePlayerUI(isBeltEquipped);
        }
    }

    [HarmonyPatch(typeof(Pickup), "PickupItem")]
    class PickupPickupItem_Patch
    {
        private static void Postfix(PickupItem pickup, bool forcePickup, ref bool triggerHandAnimation)
        {
            if (!forcePickup && !pickup.canBePickedUp)
            {
                return;
            }
            // If we have: gathered the item via Hook, Scrap Hook on hand, and a Pulley equipped, remove durability from the chestpiece.
            PlayerInventory playerInventory = RAPI.GetLocalPlayer().Inventory;
            if (playerInventory.GetSelectedHotbarItem() == null)
            {
                return;
            }
            string currentlySelectedItem = playerInventory.GetSelectedHotbarItem().UniqueName;
            if (forcePickup && !currentlySelectedItem.Equals("Hook_Plastic") && currentlySelectedItem.Contains("Hook_"))
            {
                bool isPulleyEquipped = (equippedItemsCache[CustomEquipSlotType.Get("Chest")] != null);
                if (isPulleyEquipped)
                {
                    playerInventory.RemoveDurabillityFromEquipment(EquipSlotType.Chest);
                }
            }
        }
    }

    [HarmonyPatch(typeof(PersonController), "WaterControll")]
    class PersonControllerWaterControll_Patch
    {
        private static float movingTimer = 0f;
        private static float submersedTimer = 0f;
        private static void Postfix(ref bool ___moving, ref SubmersionState ___submersionState)
        {
            if (___moving)
            {
                movingTimer += Time.deltaTime;
                if (movingTimer >= 1f)
                {
                    movingTimer = 0f;
                    string equipmentFeetCache = equippedItemsCache[CustomEquipSlotType.Get("SwimFeet")];
                    bool isFlipperEquipped = (equipmentFeetCache != null && equipmentFeetCache.Contains("Flippers"));
                    if (isFlipperEquipped)
                    {
                        PlayerInventory playerInventory = RAPI.GetLocalPlayer().Inventory;
                        Slot_Equip feetEquipmentSlot = playerInventory.GetEquipmentSlotFromEquipmentType(CustomEquipSlotType.Get("SwimFeet"));
                        feetEquipmentSlot.IncrementUses(-1);
                    }
                }
            }
            if (___submersionState == SubmersionState.Full)
            {
                submersedTimer += Time.deltaTime;
                if (submersedTimer >= 1f)
                {
                    submersedTimer = 0f;
                    string equipmentHeadCache = equippedItemsCache[CustomEquipSlotType.Get("OxygenBottle")];
                    bool isOxygenBottleEquipped = (equipmentHeadCache != null && equipmentHeadCache.Contains("Oxygen_"));
                    if (isOxygenBottleEquipped)
                    {
                        PlayerInventory playerInventory = RAPI.GetLocalPlayer().Inventory;
                        playerInventory.RemoveDurabillityFromEquipment(CustomEquipSlotType.Get("OxygenBottle"));
                    }
                }
            }
        }
    }



    [HarmonyPatch(typeof(PickupChanneling), "ProgressChannel")]
    class PickupChannelingProgressChannel_Patch
    {
        private static void Prefix(ref float ___pickupTime)
        {
            // @TODO: Not a huge deal at all since all this does is literally an assignment but figure out the one-stop shop to set the pickupTime.
            ___pickupTime = 1f - (1f * globalGatherSpeedMultiplier);
        }
    }

    [HarmonyPatch(typeof(PickupChanneling), "TakeItemNetworked")]
    class PickupChannelingTakeItemNetworked_Patch
    {
        private static void Postfix(ref Network_Player ___player)
        {
            bool isGloveEquipped = equippedItemsCache[CustomEquipSlotType.Get("Glove")] != null;
            if (isGloveEquipped)
            {
                ___player.Inventory.RemoveDurabillityFromEquipment(CustomEquipSlotType.Get("Glove"));
            }
        }
    }

    [HarmonyPatch(typeof(Axe), "OnAxeHit")]
    class AxeOnAxeHit_Patch
    {
        private static void Postfix(ref Network_Player ___playerNetwork, ref RaycastHit ___rayHit, ref LayerMask ___hitmask)
        {
            bool isGloveEquipped = equippedItemsCache[CustomEquipSlotType.Get("Glove")] != null;
            if (isGloveEquipped && Helper.HitAtCursor(out ___rayHit, 5f, ___hitmask))
            {
                if (___rayHit.transform.tag == "Tree")
                {
                    ___playerNetwork.Inventory.RemoveDurabillityFromEquipment(CustomEquipSlotType.Get("Glove"));
                }
            }
        }
    }

    [HarmonyPatch(typeof(PlayerInventory), "InitializeSlots")]
    class PlayerInventoryInitializeSlots_Patch
    {
        private static void Prefix(PlayerInventory __instance)
        {
            // @TODO: Refactor this code to split out some of the code to be neater.
            GameObject hotbarParent = GameObject.Find("Hotbar/Hotslot parent");
            Slot[] allSlots = __instance.gridLayoutGroup.GetComponentsInChildren<Slot>();
            GridLayoutGroup[] gridLayoutGroups = __instance.gridLayoutGroup.GetComponentsInChildren<GridLayoutGroup>();
            GridLayoutGroup equipmentGroup = gridLayoutGroups[1];
            Slot regularSlot = null;
            int regularSlotStartingIndex = 0;
            Slot backpackSlot = null;
            int backpackSlotStartingIndex = 0;
            Slot equipmentSlot = null;
            Slot hotbarSlot = hotbarParent.transform.GetChild(0).GetComponent<Slot>();
            foreach (Slot slot in allSlots)
            {
                if (slot.slotType == SlotType.Normal)
                {
                    regularSlot = slot;
                    regularSlotStartingIndex++;
                }
                if (slot.slotType == SlotType.Backpack)
                {
                    backpackSlot = slot;
                    backpackSlotStartingIndex++;
                }
                if (slot.slotType == SlotType.Equipment)
                {
                    equipmentSlot = slot;
                }
            }
            int hotbarSlotCount = 10;
            for (int i = 0; i < 5; ++i)
            {
                Slot clonedHotbarSlot = Instantiate(hotbarSlot);
                clonedHotbarSlot.transform.SetParent(hotbarParent.transform);
                clonedHotbarSlot.transform.localScale = new Vector3(1f, 1f, 1f);
                clonedHotbarSlot.transform.SetSiblingIndex(hotbarSlotCount);
                hotbarSlotCount++;
            }
            // Disabled for future use potentially.
            for (int i = 0; i < 0; ++i)
            {
                Slot clonedRegularSlot = Instantiate(regularSlot);
                clonedRegularSlot.transform.SetParent(__instance.gridLayoutGroup.transform);
                clonedRegularSlot.transform.localScale = new Vector3(1f, 1f, 1f);
                clonedRegularSlot.transform.SetSiblingIndex(regularSlotStartingIndex);
                regularSlotStartingIndex++;
            }
            backpackSlotStartingIndex += regularSlotStartingIndex;
            for (int i = 0; i < 10; ++i)
            {
                Slot clonedBackpackSlot = Instantiate(backpackSlot);
                clonedBackpackSlot.transform.SetParent(__instance.gridLayoutGroup.transform);
                clonedBackpackSlot.transform.localScale = new Vector3(1f, 1f, 1f);
                clonedBackpackSlot.transform.SetSiblingIndex(backpackSlotStartingIndex);
                backpackSlotStartingIndex++;
            }
            for (int i = 0; i < 5; ++i)
            {
                Slot clonedEquipmentSlot = Instantiate(equipmentSlot);
                clonedEquipmentSlot.transform.SetParent(equipmentGroup.transform);
                clonedEquipmentSlot.transform.localScale = new Vector3(1f, 1f, 1f);
            }
        }

        private static void Postfix(PlayerInventory __instance)
        {
            GameObject hotbarParent = GameObject.Find("Hotbar/Hotslot parent");
            GridLayoutGroup[] gridLayoutGroups = __instance.gridLayoutGroup.GetComponentsInChildren<GridLayoutGroup>();
            GridLayoutGroup equipmentGroup = gridLayoutGroups[1];
            Slot[] equipmentSlots = equipmentGroup.GetComponentsInChildren<Slot>();
            Slot[] hotbarSlots = hotbarParent.GetComponentsInChildren<Slot>();
            for (int i = 10; i < 15; ++i)
            {
                hotbarSlots[i].gameObject.SetActive(false);
                hotbarSlots[i].active = false;
            }
            for (int i = 5; i < 10; ++i)
            {
                equipmentSlots[i].gameObject.SetActiveSafe(false);
                equipmentSlots[i].active = false;
            }
            Transform inventoryParent = GameObject.Find("InventoryParent").transform;
            Transform inventoryPlayer = inventoryParent.Find("Inventory_Player");
            originalInventoryVector = new Vector3(inventoryPlayer.localPosition.x, inventoryPlayer.localPosition.y, 0f);
        }
    }

    [HarmonyPatch(typeof(Hotbar), "HandleHotbarSelection")]
    class HotbarHandleHotbarSelection_Patch
    {
        static void Prefix(Network_Player ___playerNetwork, ref int __state)
        {
            __state = ___playerNetwork.Inventory.hotslotCount;
            ___playerNetwork.Inventory.hotslotCount = maxHotbarSlotIndex + 1;
        }
        static void Postfix(Network_Player ___playerNetwork, int __state)
        {
            ___playerNetwork.Inventory.hotslotCount = __state;
        }
    }

    [HarmonyPatch(typeof(Inventory_ResearchTable), "LearnItem")]
    class Inventory_ResearchTableLearnItem_Patch
    {
        private static void Postfix(Inventory_ResearchTable __instance, Item_Base item, CSteamID researcherID, ref List<ResearchMenuItem> ___menuItems, ref string ___eventRef_Learn, ref NotificationManager ___notificationManager)
        {
            if (item.UniqueName == "Backpack")
            {
                // If we learn a backpack, then we want to learn all of the backpack tiers -- for the sake of not forcing players into insane material sink.
                Item_Base backpackItem = ItemManager.GetItemByName("Duffel_Backpack");
                for (int i = 0; i < ___menuItems.Count; ++i)
                {
                    if (___menuItems[i].GetItem().UniqueName == "Duffel_Backpack")
                    {
                        (___notificationManager.ShowNotification("Research") as Notification_Research).researchInfoQue.Enqueue(new Notification_Research_Info(backpackItem.settings_Inventory.DisplayName, researcherID, backpackItem.settings_Inventory.Sprite));
                        ___menuItems[i].Learn();
                    }
                }
            }
            if (item.UniqueName == "Backpack_Advanced")
            {
                // If we learn a backpack, then we want to learn all of the backpack tiers -- for the sake of not forcing players into insane material sink.
                Item_Base backpackItem = ItemManager.GetItemByName("Tactical_Backpack");
                for (int i = 0; i < ___menuItems.Count; ++i)
                {
                    if (___menuItems[i].GetItem().UniqueName == "Tactical_Backpack")
                    {
                        (___notificationManager.ShowNotification("Research") as Notification_Research).researchInfoQue.Enqueue(new Notification_Research_Info(backpackItem.settings_Inventory.DisplayName, researcherID, backpackItem.settings_Inventory.Sprite));
                        ___menuItems[i].Learn();
                    }
                }
            }
            if (item.UniqueName== "Thatch_Belt")
            {
                Item_Base[] all = ItemManager.GetAllItems().Where(belt => belt.UniqueName.Contains("_Belt")).ToArray();
                for (int i = 0; i < ___menuItems.Count; ++i)
                {
                    foreach (Item_Base one in all)
                    {
                        if (___menuItems[i].GetItem().UniqueIndex == one.UniqueIndex && one != item)
                        {
                            (___notificationManager.ShowNotification("Research") as Notification_Research).researchInfoQue.Enqueue(new Notification_Research_Info(one.settings_Inventory.DisplayName, researcherID, one.settings_Inventory.Sprite));
                            ___menuItems[i].Learn();
                        }
                    }
                }
            }
            if (item.UniqueName== "Equipment_LeatherHelmet")
            {
                Item_Base[] all = ItemManager.GetAllItems().Where(belt => belt.UniqueName.Contains("_Helmet")).ToArray();
                for (int i = 0; i < ___menuItems.Count; ++i)
                {
                    foreach (Item_Base one in all)
                    {
                        if (___menuItems[i].GetItem().UniqueIndex == one.UniqueIndex && one != item)
                        {
                            (___notificationManager.ShowNotification("Research") as Notification_Research).researchInfoQue.Enqueue(new Notification_Research_Info(one.settings_Inventory.DisplayName, researcherID, one.settings_Inventory.Sprite));
                            ___menuItems[i].Learn();
                        }
                    }
                }
            }
            if (item.UniqueName == "Equipment_LeatherChest")
            {
                Item_Base[] all = ItemManager.GetAllItems().Where(belt => belt.UniqueName.Contains("_Pulley")).ToArray();
                for (int i = 0; i < ___menuItems.Count; ++i)
                {
                    foreach (Item_Base one in all)
                    {
                        if (___menuItems[i].GetItem().UniqueIndex == one.UniqueIndex && one != item)
                        {
                            (___notificationManager.ShowNotification("Research") as Notification_Research).researchInfoQue.Enqueue(new Notification_Research_Info(one.settings_Inventory.DisplayName, researcherID, one.settings_Inventory.Sprite));
                            ___menuItems[i].Learn();
                        }
                    }
                }
            }
            if (item.UniqueName == "Equipment_LeatherLegs")
            {
                Item_Base[] all = ItemManager.GetAllItems().Where(belt => belt.UniqueName.Contains("_Boots")).ToArray();
                for (int i = 0; i < ___menuItems.Count; ++i)
                {
                    foreach (Item_Base one in all)
                    {
                        if (___menuItems[i].GetItem().UniqueIndex == one.UniqueIndex && one != item)
                        {
                            (___notificationManager.ShowNotification("Research") as Notification_Research).researchInfoQue.Enqueue(new Notification_Research_Info(one.settings_Inventory.DisplayName, researcherID, one.settings_Inventory.Sprite));
                            ___menuItems[i].Learn();
                        }
                    }
                }
            }
            if (item.UniqueName == "Worn_Gloves")
            {
                Item_Base[] all = ItemManager.GetAllItems().Where(belt => belt.UniqueName.Contains("_Gloves")).ToArray();
                for (int i = 0; i < ___menuItems.Count; ++i)
                {
                    foreach (Item_Base one in all)
                    {
                        if (___menuItems[i].GetItem().UniqueIndex == one.UniqueIndex && one != item)
                        {
                            (___notificationManager.ShowNotification("Research") as Notification_Research).researchInfoQue.Enqueue(new Notification_Research_Info(one.settings_Inventory.DisplayName, researcherID, one.settings_Inventory.Sprite));
                            ___menuItems[i].Learn();
                        }
                    }
                }
            }
            if (item.UniqueName == "Flipper")
            {
                Item_Base[] all = ItemManager.GetAllItems().Where(belt => belt.UniqueName.Contains("_Flippers")).ToArray();
                for (int i = 0; i < ___menuItems.Count; ++i)
                {
                    foreach (Item_Base one in all)
                    {
                        if (___menuItems[i].GetItem().UniqueIndex == one.UniqueIndex && one != item)
                        {
                            (___notificationManager.ShowNotification("Research") as Notification_Research).researchInfoQue.Enqueue(new Notification_Research_Info(one.settings_Inventory.DisplayName, researcherID, one.settings_Inventory.Sprite));
                            ___menuItems[i].Learn();
                        }
                    }
                }
            }
            if (item.UniqueName == "OxygenBottle")
            {
                Item_Base[] all = ItemManager.GetAllItems().Where(belt => belt.UniqueName.Contains("Oxygen_")).ToArray();
                for (int i = 0; i < ___menuItems.Count; ++i)
                {
                    foreach (Item_Base one in all)
                    {
                        if (___menuItems[i].GetItem().UniqueIndex == one.UniqueIndex &&one!=item)
                        {
                            (___notificationManager.ShowNotification("Research") as Notification_Research).researchInfoQue.Enqueue(new Notification_Research_Info(one.settings_Inventory.DisplayName, researcherID, one.settings_Inventory.Sprite));
                            ___menuItems[i].Learn();
                        }
                    }
                }
            }
        }
    }

    [HarmonyPatch(typeof(Inventory_ResearchTable), "OnWorldRecievedLate")]
    class Inventory_ResearchTableOnWorldRecievedLate_Patch
    {
        private static void Postfix(Inventory_ResearchTable __instance)
        {
            if (Raft_Network.IsHost)
            {
                List<ResearchMenuItem> menuItems = __instance.GetMenuItems();
                bool backpackAlreadyResearched = menuItems.Exists(menuItem => (menuItem.GetItem().UniqueName.Contains("Backpack") && menuItem.Learned));
                if (backpackAlreadyResearched)
                {
                    Item_Base[] allBackpacks = ItemManager.GetAllItems().Where(backpack => backpack.UniqueName.Contains("_Backpack")).ToArray();
                    for (int i = 0; i < menuItems.Count; ++i)
                    {
                        foreach (Item_Base backpackItem in allBackpacks)
                        {
                            if (menuItems[i].GetItem().UniqueIndex == backpackItem.UniqueIndex)
                            {
                                menuItems[i].Learn();
                            }
                        }
                    }
                }
                bool beltAlreadyResearched = menuItems.Exists(menuItem => (menuItem.GetItem().UniqueName.Contains("Belt") && menuItem.Learned));
                if (beltAlreadyResearched)
                {
                    Item_Base[] allBelts = ItemManager.GetAllItems().Where(backpack => backpack.UniqueName.Contains("_Belt")).ToArray();
                    for (int i = 0; i < menuItems.Count; ++i)
                    {
                        foreach (Item_Base beltItem in allBelts)
                        {
                            if (menuItems[i].GetItem().UniqueIndex == beltItem.UniqueIndex)
                            {
                                menuItems[i].Learn();
                            }
                        }
                    }
                }
            }
        }
    }

    [HarmonyPatch(typeof(PlayerInventory), "FindSuitableSlot")]
    static class PlayerInventoryFindSuitableSlot_Patch
    {
        static IEnumerable<CodeInstruction> Transpiler(IEnumerable<CodeInstruction> instructions)
        {
            var code = instructions.ToList();
            var lbl = (Label)code[code.FindIndex(code.FindIndex(x => x.operand is MethodInfo m && m.Name == "get_Count"), x => x.operand is Label)].operand;
            code.InsertRange(code.FindIndex(x => x.opcode == OpCodes.Stloc_3) + 1, new[]
            {
                new CodeInstruction(OpCodes.Ldloc_3),
                new CodeInstruction(OpCodes.Call,AccessTools.Method(typeof(PlayerInventoryFindSuitableSlot_Patch),nameof(IsAcceptableSlot))),
                new CodeInstruction(OpCodes.Brfalse,lbl)
            });
            return code;
        }
        static bool IsAcceptableSlot(Slot slot) => slot && slot.active;
    }

    [ConsoleCommand(name: "backpack", docs: "remove equipped backpack")]
    public static string MyCommand(string[] args)
    {
        RAPI.GetLocalPlayer().Inventory.equipSlots.ForEach(a =>
        {
            if (a.itemInstance.settings_equipment.EquipType == EquipSlotType.Backpack)
                RAPI.GetLocalPlayer().Inventory.DropItem(a);
        });
        return "OK";
    }

    public void OnModUnload()
    {
        harmony.UnpatchAll(instanceId);
        UnloadEquipment();
        Debug.Log("Mod AugmentedEquipment has been unloaded!");
    }
}
